# -*- encoding: utf-8 -*-

from .element import element
from .tabular import tabular

import itertools
import pandas as pd


class table(element):
    """ Class representing a latex figure. """

    templates = {
        'caption': '\\caption{{{content}}}',
        'content': """\\begin{{table}}[{options}]
{centered}
{inner}
{caption}{label}
\\end{{table}}"""
    }

    autolabel = False

    def __init__(self, childrens=None, parent=None,
                 caption=None, options='!ht',
                 centered=True, label=None):
        """ Create a table with given options.

        Parameters:
          - options Options for the table.
          - centered Set to true to add a centering command.
        """
        super().__init__(parent, childrens, label=label)
        self.options = options
        self.caption = caption
        self.centered = ''
        if centered:
            self.centered = '\\centering'

    def content(self):
        caption = ''
        if self.caption:
            caption = self.fmt().format('caption', {'content': self.caption})
        return self.fmt().format('content', {
            'options': self.options,
            'centered': self.centered,
            'caption': caption,
            'label': self.format_label(self.label),
            'inner': super().content()
        })


def multicolumn(n, s=''):
    if n == 1:
        return s
    return '\\multicolumn{' + str(n) + '}{c}{' + s + '}'


class tabledf(table):

    def __init__(self, df, header=None, multilevels=False, fmt=None, **kargs):
        super().__init__(**kargs)
        if header is None:
            header = self.create_header(df)
            multilevels = True
            df = df.reset_index()
        if multilevels:
            ncols = len(header[-1])
        else:
            ncols = len(header)
        tab = tabular(columns='r' * ncols)
        if multilevels:
            tab.addrows(header)
        else:
            tab.addrow(header)
        tab.addhline()
        tab.addrows(df.as_matrix(), fmt=fmt)
        self.add(tab)

    def create_header(self, df):
        cols = df.columns
        if not isinstance(cols, pd.MultiIndex):
            return df.columns
        idx = df.index.names
        header = []
        for levels, labels in zip(cols.levels, cols.labels):
            tmp = [multicolumn(len(idx))]
            for i, g in itertools.groupby(labels):
                tmp.append(multicolumn(len(list(g)), levels[i]))
            header.append(tmp)
        header[-1] = idx + header[-1][1:]  # Update last row
        return header
